class OutputProperties:
    MESSAGE_PROPERTY_NAME: str = "message"
    TIMESTAMP_PROPERTY_NAME: str = "timestamp"
    LEVEL_PROPERTY_NAME: str = "level"
    NEW_LINE_PROPERTY_NAME: str = "new_line"
    EXCEPTION_PROPERTY_NAME: str = "exception"
    PROPERTIES_PROPERTY_NAME: str = "properties"
